### react-scrollwatcher
#### installation
```javascript
npm install react-scrollwatcher
```
#### usage

##### example 1
```javascript
    import React, {useRef} from "react"
    import ScrollWatcher from "react-scrollwatcher"

    function SomeComponent({children}){
      const watchedNode = useRef()
      const onOffsetChange = (matrix) => {
        console.dir(matrix)
      }

      return (
        <ScrollWatcher watch={watchedNode} onOffsetChange={onOffsetChange}>
            <div ref={watchedNode}>
                {children}
              </div>
          </ScrollWatcher>
      )
    }
```

##### example 2
```javascript
    import React, {forwardRef, useContext, useRef} from "react"
    import ScrollWatcher, {useScrollWatch} from "react-scrollwatcher"

    const WatchedComponent = forwardRef(({children}, ref) => {
      const matrix = useScrollWatch()
      console.log(matrix)

      return (
          <div ref={ref}>
            {children}
          </div>
        )
    })

    function SomeComponent({children}){
      const watchedNode = useRef()
      const onOffsetChange = (matrix) =>
        console.dir(matrix)

      return (
        <ScrollWatcher watch={watchedNode} onOffsetChange={onOffsetChange}>
          <WatchedComponent ref={watchedNode}>
              {children}
            </WatchedComponent>
          </ScrollWatcher>
        )
    }
```

#### watching a node inside another scrollable node
Only supports one level of nesting.

##### example 3
```javascript
    import React, {useRef} from "react"
    import ScrollWatcher from "react-scrollwatcher"

    function SomeComponent({children}){
      const scrollableNode = useRef()
      const watchedNode = useRef()
      const onOffsetChange = (matrix) =>
        console.dir(matrix)

      return (
        <ScrollWatcher watch={watchedNode} inside={scrollableNode} onOffsetChange={onOffsetChange}>
          <div ref={scrollableNode}>
            <div ref={watchedNode}>
              {children}
            </div>
          <div ref={watchedNode}>
        </ScrollWatcher>
      )
    }
```


#### scrollwatcher matrix

When the watched node is not visible, matrix is **undefined**, otherwise it is an object with the following properties:

**coverage**:Number [0-1] *% of the node visible inside the viewport*

**coverageX**:Number [0-1] *% of the node visible in the viewport on the horizontal plane*

**coverageY**:Number  [0-1] *% of the node visible in the viewport on the vertical plane*

**deltaX**:Number *the distance in pixels the node moved since the last matrix update on the horizontal plane*

**deltaY**:Number *the distance in pixels the node moved since the last matrix update on the vertical plane*

**direction**:bitmask *directions in bitmask format ( see directions )*

**offsetX**:Number *distance in pixels from the left side of the viewport to the left side of node*

**offsetY**:Number *distance in pixels from the top side of the viewport to the top of the node*

**overflowX**:Number *distance in pixels from the right side of the viewport to the right side of the node*

**overflowY**:Number *distance in pixels from the bottom side of the viewport to the bottom side of the node*

**progressX**:Number [0,1] *% representing the distance travelled by the node towards the point when the right side of the node has reached the left side of the viewport*

**progressY**:Number [0,1] *% representing the distance travelled by the node towards the point when the bottom side of the node has reached the top side of the viewport*

**visibility**:Boolean *always true when reading the matrix object*

**height**:Number *node.clientHeight*

**contentHeight**:Number *node.scrollHeight*

**visibleHeight**:Number *absolute value in pixels of how much of the node is visible on the vertical plane*

**width**:Number *node.clientWidth*

**contentWidth**:Number *node.scrollWidth*

**visibleWidth**:Number *absolute value in pixels of how much of the node is visible on the horizontal plane*


#### directions

The directions towards which the node is moving relative to the scroll of the window are given in a bitmask format.

```javascript
    import ScrollWatcher, {directions} from "react-scrollwatcher"

    function onOffsetChange(matrix){
      matrix.direction & directions.UP
      matrix.direction & directions.DOWN
      matrix.direction & directions.LEFT
      matrix.direction & directions.RIGHT
      matrix.direction & (directions.UP^direction.LEFT)
      matrix.direction & (directions.UP^direction.RIGHT)
      matrix.direction & (directions.DOWN^direction.LEFT)
      matrix.direction & (directions.DOWN^direction.RIGHT)
    }

```

#### notes

The position of the watched node is computed at:
*	DOM events:
	*	focus
	*	orientationchange
	*	resize
	*	scroll
*	DOM Mutations
*	watcher creation.

Listeners are only active when at least one node is being actively watched.
