"use strict"

var path = require("path")
var isWsl = require('is-wsl')
var TerserPlugin = require('terser-webpack-plugin')

var production = process.env.NODE_ENV === "production"
var appDist = path.resolve(process.cwd(), "./dist")
var appSrc = path.resolve(process.cwd(), "./src")

module.exports = {
    entry: "./src/index.js"
  , devtool: !production ? "cheap-module-source-map" : "source-map"
  , output: {
        path: appDist
      , filename: "react-scrollwatcher.js"
      , libraryTarget: "umd"
      , library: "react-scrollwatcher"
    }
  , optimization: {
        minimize: production
      , minimizer: [
            new TerserPlugin({
                terserOptions: {
                   parse: { ecma: 8}
                , compress: {
                      ecma: 5
                    , warnings: false
                    , comparisons: false
                    , inline: 2
                  }
                , mangle: { safari10: true }
                , output: {
                      ecma: 5
                    , comments: false
                    , ascii_only: true
                  }
                }
              , parallel: !isWsl
              , cache: true
              , sourceMap: true
            })
        ]
    }
  , externals: {
        react: { commonjs: "react", commonjs2: "react", amd: "React", root: "React" }
      , "react-dom": { commonjs: "react-dom", commonjs2: "react-dom", amd: "ReactDOM", root: "ReactDOM" }
    }
  , module: {
        strictExportPresence: true
      , rules: [
            { parser: { requireEnsure: false } }
          , {
              test: /\.(js)$/
            , enforce: 'pre'
            , use: [{
                options: {
                    formatter: require.resolve('react-dev-utils/eslintFormatter')
                  , eslintPath: require.resolve('eslint')
                }
              , loader: require.resolve('eslint-loader')
              }]
            , include: appSrc,
            }
          , {
                test: /\.(js)$/
              , include: appSrc
              , loader: require.resolve("babel-loader"),
            options: {
                customize: require.resolve("babel-preset-react-app/webpack-overrides")
              , plugins: []
              , cacheDirectory: true
              , cacheCompression: production
              , compact: production
            }
          }
        ]
    }
}
